/*
 * DocumentConverter.java
 *
 * Created on 2009.02.04 - 19:00:07
 *
 */

package com.example;

import com.sun.star.frame.XDesktop;
import com.sun.star.uno.UnoRuntime;

/**
 *
 * @author mk
 */
public class DocumentConverter {
    
	/**
	 *  .  .
	 */
	public short pageWidth	= 0;

	/**
	 *  .  .
	 */
	public short pageHeight	= 0;

	/**
	 *   .  
	 */
	public short pageMargin	= 0;

	/**
	 *    . ,   
	 */
	public String sourceFile	= new String("");

	/**
	 *    . PDF.
	 */
	public String destFile		= new String("");

	/**
	 *    -   
	 */
	public String documentType	= new String("text");

	/**
	 *  
	 */
	public com.sun.star.frame.XDesktop xDesktop = null;

	/**
	 *  
	 */
	public com.sun.star.lang.XComponent xComponent = null;




    /** Creates a new instance of DocumentConverter */
    public DocumentConverter() {		
    }




    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

		// **    

		//   
		boolean success = new Boolean(false);

		//  
		DocumentConverter converter = new DocumentConverter();

		// **  
		
		//    , 
		if(args.length == 0) {
			System.out.println("Usage: java -jar DocumentConverter.jar -w=WIDTH -h=HEIGHT -m=MARGIN -s=SOURCE -d=DESTINATION -t=TYPE");
			System.out.println("Params:");
			System.out.println("       WIDTH, HEIGHT, MARGIN - dimensions in millimeters.");
			System.out.println("       SOURCE - document to convert (TXT,DOC,RTF,ODT,HTML,XML,JPG,PNG,GIF).");
			System.out.println("       DESTINATION - PDF-document name.");
			System.out.println("       TYPE - 'text' for text documents, 'picture' for pictures. Default 'text'.");
                        System.out.println("       Developed in M.Kechinov's web-development studio: mkechinov.com");
			System.exit(0);
		}

		//  
		for( int i = 0; i < args.length; i++ ) {

			//  
			if( args[i].substring(0, 3).equals("-w=") ) {
				//  
				try {
					converter.pageWidth = Short.parseShort( args[i].substring(3, args[i].length()), 10 );
				} catch (java.lang.Exception e){
					System.err.println("Bad WIDTH value: " + converter.pageWidth);
					System.exit(1);
				}
				//      , 
				if( converter.pageWidth < 100 || converter.pageWidth > 500 ) {
					System.err.println("WIDTH is too big or small. Must be between 100 and 500 mm.");
					System.exit(1); 
				}
			}

			//  
			if( args[i].substring(0, 3).equals("-h=") ) {
				//  
				try {
					converter.pageHeight = Short.parseShort( args[i].substring(3, args[i].length()), 10 );
				} catch (java.lang.Exception e){
					System.err.println("Bad HEIGHT value: " + converter.pageHeight);
					System.exit(1);
				}
				//      , 
				if( converter.pageHeight < 100 || converter.pageHeight > 500 ) {
					System.err.println("HEIGHT is too big or small. Must be between 100 and 500 mm.");
					System.exit(1);
				}
			}

			// ,    
			if( args[i].substring(0, 3).equals("-m=") ) {
				//  
				try {
					converter.pageMargin = Short.parseShort( args[i].substring(3, args[i].length()), 10 );
				} catch (java.lang.Exception e){
					System.err.println("Bad MARGIN value: " + converter.pageMargin);
					System.exit(1);
				}
				//      , 
				if( converter.pageMargin < 0 || converter.pageMargin > 25 ) {
					System.err.println("MARGIN is too big or small. Must be between 0 and 25 mm.");
					System.exit(1);
				}
			}

			//   
			if( args[i].substring(0, 3).equals("-s=") ) {
				//  
				try {
					converter.sourceFile = args[i].substring(3, args[i].length());
				} catch (java.lang.Exception e){
					System.err.println("Bad SOURCE value.");
					System.exit(1);
				}
				//    , 
				if( converter.sourceFile.length() < 5 ) {
					System.err.println("SOURCE path is wrong.");
					System.exit(1);
				}
			}

			//   
			if( args[i].substring(0, 3).equals("-d=") ) {
				//  
				try {
					converter.destFile = args[i].substring(3, args[i].length());
				} catch (java.lang.Exception e){
					System.err.println("Bad DESTINATION value.");
					System.exit(1);
				}
				//    , 
				if( converter.destFile.length() < 5 ) {
					System.err.println("DESTINATION path is wrong.");
					System.exit(1);
				}
			}

			//     (  )
			if( args[i].substring(0, 3).equals("-t=") ) {
				//  
				try {
					converter.documentType = args[i].substring(3, args[i].length());
				} catch (java.lang.Exception e){
					System.err.println("Bad TYPE value.");
					System.exit(1);
				}
				//    ,    "",  ""
				if( converter.documentType.equals("picture") ) {
					converter.documentType = new String("picture");
				} else {
					converter.documentType = new String("text");
				}
			}
			
		}

		//    , 
		if( converter.sourceFile.equals("") ) {
			System.err.println("SOURCE is not defined.");
			System.exit(1);
		}
		//     , 
		if( converter.destFile.equals("") ) {
			System.err.println("DESTINATION is not defined.");
			System.exit(1);
		}

		//        OpenOffice (x100)
		converter.pageHeight	*= 100;
		converter.pageWidth		*= 100;
		converter.pageMargin	*= 100;

		// ** 

		//   
		success = converter.getDesktop();

		//    , 
		if( success != true ) {
			System.err.println("Could not connect to OpenOffice.");
			System.exit(1);
		}

		//  
		if( converter.documentType.equals("text") ) {
			success = converter.convertDocument();
		} else {
			success = converter.convertPicture();
		}

		//    , 
		if( success == false ) {
			System.err.println("Could not convert document.");
			System.exit(1);
		}

		//   
		System.exit(0);

    }




	/**
	 *    OpenOffice,       boolean
	 *
	 * @return boolean
	 */
	public boolean getDesktop() {

		com.sun.star.lang.XMultiComponentFactory xMCF = null;

		try {

			// Get the remote office component context
			com.sun.star.uno.XComponentContext xContext = null;
			xContext = com.sun.star.comp.helper.Bootstrap.bootstrap();

			// Get the remote office service manager
			xMCF = xContext.getServiceManager();

			// Check, if not have service
			if( xContext == null ) {
				System.err.println("ERROR: Could not bootstrap default Office.");
			}

			// Get desktop
			Object oDesktop = xMCF.createInstanceWithContext("com.sun.star.frame.Desktop", xContext);
			this.xDesktop = (com.sun.star.frame.XDesktop) UnoRuntime.queryInterface(com.sun.star.frame.XDesktop.class, oDesktop);

        }
        catch (java.lang.Exception e){
            e.printStackTrace();
			System.exit(1); 
        }

		//  
		return true;

	}



	public boolean convertDocument() {

		// **  
		
		com.sun.star.frame.XComponentLoader xComponentLoader = null;

		com.sun.star.beans.PropertyValue openProperties[] = new com.sun.star.beans.PropertyValue[1];
		openProperties[0] = new com.sun.star.beans.PropertyValue();
		openProperties[0].Name = "Hidden";
		openProperties[0].Value = new Boolean(true);


		try {
			xComponentLoader = (com.sun.star.frame.XComponentLoader)UnoRuntime.queryInterface(com.sun.star.frame.XComponentLoader.class, this.xDesktop);
			this.xComponent = xComponentLoader.loadComponentFromURL(this.sourceFile, "_blank", 0, openProperties);
		} catch( Exception e) {
			System.err.println("Could not open source file.");
            e.printStackTrace(System.err);
			return false;
        }

		// **   
		try {
			com.sun.star.text.XTextDocument xTextDocument = (com.sun.star.text.XTextDocument) UnoRuntime.queryInterface(com.sun.star.text.XTextDocument.class, this.xComponent);
			com.sun.star.text.XText xText = xTextDocument.getText();
			com.sun.star.text.XTextCursor xTextCursor = xText.createTextCursor();
			com.sun.star.beans.XPropertySet xTextCursorProps = (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, xTextCursor);
			String pageStyleName = xTextCursorProps.getPropertyValue("PageStyleName").toString();
			com.sun.star.style.XStyleFamiliesSupplier xSupplier = (com.sun.star.style.XStyleFamiliesSupplier) UnoRuntime.queryInterface(com.sun.star.style.XStyleFamiliesSupplier.class, xTextDocument);
			com.sun.star.container.XNameAccess xFamilies = (com.sun.star.container.XNameAccess) UnoRuntime.queryInterface(com.sun.star.container.XNameAccess.class, xSupplier.getStyleFamilies());
			com.sun.star.container.XNameContainer xFamily = (com.sun.star.container.XNameContainer) UnoRuntime.queryInterface(com.sun.star.container.XNameContainer.class, xFamilies.getByName("PageStyles"));
			com.sun.star.style.XStyle xStyle = (com.sun.star.style.XStyle) UnoRuntime.queryInterface(com.sun.star.style.XStyle.class, xFamily.getByName(pageStyleName));
			com.sun.star.beans.XPropertySet xStyleProps = (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, xStyle);
			xStyleProps.setPropertyValue("LeftMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("RightMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("BottomMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("TopMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("Height",new Short(this.pageHeight));
			xStyleProps.setPropertyValue("Width",new Short(this.pageWidth));
		} catch( Exception e) {
			System.err.println("Could not change style.");
            e.printStackTrace(System.err);
			return false;
        }

		
		// **  
		
		com.sun.star.frame.XStorable xStorable = (com.sun.star.frame.XStorable) UnoRuntime.queryInterface(com.sun.star.frame.XStorable.class, this.xComponent);
		com.sun.star.beans.PropertyValue storeProperties[] = new com.sun.star.beans.PropertyValue[1];

		storeProperties[0] = new com.sun.star.beans.PropertyValue();
		storeProperties[0].Name = "FilterName";
		storeProperties[0].Value = "writer_pdf_Export";

		try {
			xStorable.storeToURL(this.destFile, storeProperties);
		} catch( Exception e) {
			System.err.println("Could not save destination file.");
			e.printStackTrace(System.err);
			//  ,     
        }

		// Closing the converted document
		com.sun.star.util.XCloseable xCloseable = (com.sun.star.util.XCloseable) UnoRuntime.queryInterface(com.sun.star.util.XCloseable.class, xStorable);

		if ( xCloseable != null ) {
			try {
				xCloseable.close(false);
			} catch( Exception e) {
				System.err.println("Could not close document, tryout 1.");
				e.printStackTrace(System.err);
				return false;
			}
		} else {
			try {
				com.sun.star.lang.XComponent xComponentDispose = ( com.sun.star.lang.XComponent ) UnoRuntime.queryInterface(com.sun.star.lang.XComponent.class, xStorable);
				xComponentDispose.dispose();
			} catch( Exception e) {
				System.err.println("Could not close document, tryout 2.");
				e.printStackTrace(System.err);
				return false;
			}
		}

		return true;
	}




	public boolean convertPicture() {

		// **  

		com.sun.star.frame.XComponentLoader xComponentLoader = null;

		com.sun.star.beans.PropertyValue openProperties[] = new com.sun.star.beans.PropertyValue[1];
		openProperties[0] = new com.sun.star.beans.PropertyValue();
		openProperties[0].Name = "Hidden";
		openProperties[0].Value = new Boolean(true);

		try {
			xComponentLoader = (com.sun.star.frame.XComponentLoader)UnoRuntime.queryInterface(com.sun.star.frame.XComponentLoader.class, this.xDesktop);
			this.xComponent = xComponentLoader.loadComponentFromURL("private:factory/swriter", "_blank", 0, openProperties);
		} catch( Exception e) {
			System.err.println("Could not create blank file.");
            e.printStackTrace(System.err);
			return false;
        }

		//    
		com.sun.star.text.XTextDocument xTextDocument = (com.sun.star.text.XTextDocument) UnoRuntime.queryInterface(com.sun.star.text.XTextDocument.class, this.xComponent);
		com.sun.star.text.XText xText = xTextDocument.getText();
		com.sun.star.text.XTextCursor xTextCursor = xText.createTextCursor();
		com.sun.star.beans.XPropertySet xTextCursorProps = (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, xTextCursor);

		// **  
		com.sun.star.lang.XMultiServiceFactory xMSFDoc = (com.sun.star.lang.XMultiServiceFactory)UnoRuntime.queryInterface(com.sun.star.lang.XMultiServiceFactory.class, xTextDocument);
		Object oGraphic = null;
		try {
			// Creating the service GraphicObject
			oGraphic = xMSFDoc.createInstance("com.sun.star.text.TextGraphicObject");
		} catch ( Exception e ) {
			System.out.println( "Could not create instance of TextGraphicObject" );
			e.printStackTrace(System.err);
			return false;
		}
		com.sun.star.text.XTextContent xTextContent = (com.sun.star.text.XTextContent)UnoRuntime.queryInterface(com.sun.star.text.XTextContent.class, oGraphic );
		try {
			xText.insertTextContent(xTextCursor, xTextContent, true);
		} catch ( Exception e ) {
			System.out.println( "Could not insert image" );
			e.printStackTrace(System.err);
			return false;
		}
		com.sun.star.beans.XPropertySet xPropSet = (com.sun.star.beans.XPropertySet)UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, oGraphic);
		try {
			xPropSet.setPropertyValue( "GraphicURL", this.sourceFile );
		} catch ( Exception e ) {
			System.out.println( "Could not insert image" );
			e.printStackTrace(System.err);
			return false;
		}

		// **   
		try {
			String pageStyleName = xTextCursorProps.getPropertyValue("PageStyleName").toString();
			com.sun.star.style.XStyleFamiliesSupplier xSupplier = (com.sun.star.style.XStyleFamiliesSupplier) UnoRuntime.queryInterface(com.sun.star.style.XStyleFamiliesSupplier.class, xTextDocument);
			com.sun.star.container.XNameAccess xFamilies = (com.sun.star.container.XNameAccess) UnoRuntime.queryInterface(com.sun.star.container.XNameAccess.class, xSupplier.getStyleFamilies());
			com.sun.star.container.XNameContainer xFamily = (com.sun.star.container.XNameContainer) UnoRuntime.queryInterface(com.sun.star.container.XNameContainer.class, xFamilies.getByName("PageStyles"));
			com.sun.star.style.XStyle xStyle = (com.sun.star.style.XStyle) UnoRuntime.queryInterface(com.sun.star.style.XStyle.class, xFamily.getByName(pageStyleName));
			com.sun.star.beans.XPropertySet xStyleProps = (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, xStyle);
			xStyleProps.setPropertyValue("LeftMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("RightMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("BottomMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("TopMargin",new Short(this.pageMargin));
			xStyleProps.setPropertyValue("Height",new Short(this.pageHeight));
			xStyleProps.setPropertyValue("Width",new Short(this.pageWidth));
		} catch( Exception e) {
			System.err.println("Could not change style.");
            e.printStackTrace(System.err);
			return false;
        }


		// **  

		com.sun.star.frame.XStorable xStorable = (com.sun.star.frame.XStorable) UnoRuntime.queryInterface(com.sun.star.frame.XStorable.class, this.xComponent);
		com.sun.star.beans.PropertyValue storeProperties[] = new com.sun.star.beans.PropertyValue[1];

		storeProperties[0] = new com.sun.star.beans.PropertyValue();
		storeProperties[0].Name = "FilterName";
		storeProperties[0].Value = "writer_pdf_Export";

		try {
			xStorable.storeToURL(this.destFile, storeProperties);
		} catch( Exception e) {
			System.err.println("Could not save destination file.");
			e.printStackTrace(System.err);
			//  ,     
        }

		// Closing the converted document
		com.sun.star.util.XCloseable xCloseable = (com.sun.star.util.XCloseable) UnoRuntime.queryInterface(com.sun.star.util.XCloseable.class, xStorable);

		if ( xCloseable != null ) {
			try {
				xCloseable.close(false);
			} catch( Exception e) {
				System.err.println("Could not close document, tryout 1.");
				e.printStackTrace(System.err);
				return false;
			}
		} else {
			try {
				com.sun.star.lang.XComponent xComponentDispose = ( com.sun.star.lang.XComponent ) UnoRuntime.queryInterface(com.sun.star.lang.XComponent.class, xStorable);
				xComponentDispose.dispose();
			} catch( Exception e) {
				System.err.println("Could not close document, tryout 2.");
				e.printStackTrace(System.err);
				return false;
			}
		}

		return true;
	}




}
